@@ -241,6 +241,9 @@ NULL
 #' `extra_time` can also be used to fill in missing time steps for the purposes
 #' of a random walk or AR(1) process if the gaps between time steps are uneven.
 #'
+#' `extra_time` can include only extra time steps or all time steps including
+#' those found in the fitted data. This latter option may be simpler.
+#'
 #' **Regularization and priors**
 #'
 #' You can achieve regularization via penalties (priors) on the fixed effect
@@ -775,19 +778,7 @@ sdmTMB <- function(
     offset <- data[[offset]]
   }
 
-  if (!is.null(extra_time)) { # for forecasting or interpolating
-    data <- expand_time(df = data, time_slices = extra_time, time_column = time,
-      weights = weights, offset = offset, upr = upr)
-    offset <- data[["__sdmTMB_offset__"]] # expanded
-    weights <- data[["__weight_sdmTMB__"]] # expanded
-    upr <- data[["__dcens_upr__"]] # expanded
-    spde$loc_xy <- as.matrix(data[,spde$xy_cols,drop=FALSE])
-    spde$A_st <- fmesher::fm_basis(spde$mesh, loc = spde$loc_xy)
-    spde$sdm_spatial_id <- seq(1, nrow(data)) # FIXME?
-  } else {
-    data[["__fake_data__"]] <- FALSE
-  }
-  check_irregalar_time(data, time, spatiotemporal, time_varying)
+  check_irregalar_time(data, time, spatiotemporal, time_varying, extra_time = extra_time)
 
   spatial_varying_formula <- spatial_varying # save it
   if (!is.null(spatial_varying)) {
@@ -804,7 +795,6 @@ sdmTMB <- function(
     if (length(attr(z_i, "contrasts")) && !.int && !omit_spatial_intercept) { # factors with ~ 0 or ~ -1
       msg <- c("Detected predictors with factor levels in `spatial_varying` with the intercept omitted from the `spatial_varying` formula.",
         "You likely want to set `spatial = 'off'` since the constant spatial field (`omega_s`) also represents a spatial intercept.`")
-        # "As of version 0.3.1, sdmTMB turns off the constant spatial field `omega_s` when `spatial_varying` is specified so that the intercept or factor-level means are fully described by the spatially varying random fields `zeta_s`.")
       cli_inform(paste(msg, collapse = " "))
     }
     .int <- grep("(Intercept)", colnames(z_i))
@@ -1052,7 +1042,8 @@ sdmTMB <- function(
   X_ij_list <- list()
   for (i in seq_len(n_m)) X_ij_list[[i]] <- X_ij[[i]]
 
-  n_t <- length(unique(data[[time]]))
+  time_df <- make_time_lu(data[[time]], full_time_vec = union(data[[time]], extra_time))
+  n_t <- nrow(time_df)
 
   random_walk <- if (!is.null(time_varying)) switch(time_varying_type, rw = 1L, rw0 = 2L, ar1 = 0L) else 0L
   tmb_data <- list(
@@ -1064,7 +1055,7 @@ sdmTMB <- function(
     A_st       = spde$A_st,
     sim_re     = if ("sim_re" %in% names(experimental)) as.integer(experimental$sim_re) else rep(0L, 6),
     A_spatial_index = spde$sdm_spatial_id - 1L,
-    year_i     = make_year_i(data[[time]]),
+    year_i     = time_df$year_i[match(data[[time]], time_df$time_from_data)],
     ar1_fields = ar1_fields,
     simulate_t = rep(1L, n_t),
     rw_fields =  rw_fields,
@@ -1105,6 +1096,8 @@ sdmTMB <- function(
     proj_year  = 0, # dummy
     proj_spatial_index = 0, # dummy
     proj_z_i = matrix(0, nrow = 1, ncol = n_m), # dummy
+    indexes_total = numeric(0), # dummy
+    n_integration = 0L, # dummy
     spde_aniso = make_anisotropy_spde(spde, anisotropy),
     spde       = get_spde_matrices(spde),
     barrier = as.integer(barrier),
@@ -1155,9 +1148,9 @@ sdmTMB <- function(
     ln_kappa   = matrix(0, 2L, n_m),
     # ln_kappa   = rep(log(sqrt(8) / median(stats::dist(spde$mesh$loc))), 2),
     thetaf     = 0,
-    gengamma_Q = 1, # Not defined at exactly 0
+    gengamma_Q = 0.5, # Not defined at exactly 0
     logit_p_mix = 0,
-    log_ratio_mix = 0,
+    log_ratio_mix = -1, # ratio is 1 + exp(log_ratio_mix) so 0 would start fairly high
     ln_phi     = rep(0, n_m),
     ln_tau_V   = matrix(0, ncol(X_rw_ik), n_m),
     rho_time_unscaled = matrix(0, ncol(X_rw_ik), n_m),
@@ -1187,17 +1180,7 @@ sdmTMB <- function(
   tmb_map$b_j <- NULL
   if (delta) tmb_map$b_j2 <- NULL
   if (family$family[[1]] == "tweedie") tmb_map$thetaf <- NULL
-  if ("gengamma" %in% family$family) tmb_map$gengamma_Q <- NULL
-  if (family$family[[1]] %in% c("gamma_mix", "lognormal_mix", "nbinom2_mix")) {
-    tmb_map$log_ratio_mix <- NULL
-    tmb_map$logit_p_mix <- NULL
-  }
-  if (delta) {
-    if(family$family[[2]] %in% c("gamma_mix", "lognormal_mix", "nbinom2_mix")) {
-      tmb_map$log_ratio_mix <- NULL
-      tmb_map$logit_p_mix <- NULL
-    }
-  }
+  if ("gengamma" %in% family$family) tmb_map$gengamma_Q <- factor(NA)
   tmb_map$ln_phi <- rep(1, n_m)
   if (family$family[[1]] %in% c("binomial", "poisson", "censored_poisson"))
     tmb_map$ln_phi[1] <- factor(NA)
@@ -1219,7 +1202,6 @@ sdmTMB <- function(
 
   if (multiphase && is.null(previous_fit) && do_fit) {
 
-
     original_tmb_data <- tmb_data
     # much faster on first phase!?
     tmb_data$no_spatial <- 1L
@@ -1232,6 +1214,7 @@ sdmTMB <- function(
 
     tmb_obj1 <- TMB::MakeADFun(
       data = tmb_data, parameters = tmb_params,
+      profile = control$profile,
       map = tmb_map, DLL = "sdmTMB", silent = silent)
     lim <- set_limits(tmb_obj1, lower = lower, upper = upper, silent = TRUE)
 
@@ -1355,13 +1338,22 @@ sdmTMB <- function(
   }
 
   if (anisotropy && delta && !"ln_H_input" %in% names(map)) {
-    tmb_map$ln_H_input <- factor(c(1, 2, 1, 2)) # share anistropy as in VAST
+    tmb_map$ln_H_input <- factor(c(1, 2, 1, 2)) # share anisotropy as in VAST
+  }
+
+  if (family$family[[1]] %in% c("gamma_mix", "lognormal_mix", "nbinom2_mix")) {
+    tmb_map$log_ratio_mix <- NULL # performs better starting this in 2nd phase
+    tmb_map$logit_p_mix <- NULL # performs better starting this in 2nd phase
+  }
+  if (delta) {
+    if (family$family[[2]] %in% c("gamma_mix", "lognormal_mix", "nbinom2_mix")) {
+      tmb_map$log_ratio_mix <- NULL
+      tmb_map$logit_p_mix <- NULL
+    }
   }
 
   if (tmb_data$threshold_func > 0) tmb_map$b_threshold <- NULL
-
-  if (control$profile && delta)
-    cli_abort("Profile not yet working with delta models.")
+  if ("gengamma" %in% family$family) tmb_map$gengamma_Q <- NULL
 
   for (i in seq_along(map)) { # user supplied
     cli_inform(c(i = paste0("Fixing or mirroring `", names(map)[i], "`")))
@@ -1372,19 +1364,20 @@ sdmTMB <- function(
   }
   tmb_map <- c(map, tmb_map) # add user-supplied mapping
 
-  prof <- c("b_j")
-  if (delta) prof <- c(prof, "b_j2")
+  if (isTRUE(control$profile)) {
+    prof <- c("b_j")
+    if (delta) prof <- c(prof, "b_j2")
+    control$profile <- prof
+  } else if (is.character(control$profile)) {
+    prof <- control$profile
+    control$profile <- prof
+  } else {
+    control$profile <- NULL
+  }
 
-  lu <- make_year_lu(data[[time]])
-  fd <- data[['__fake_data__']]
-  tmp <- data[!fd,,drop=FALSE]
-  # strip fake data from A matrix:
-  if (sum(fd) > 0L) spde <- make_mesh(tmp, spde$xy_cols, mesh = spde$mesh)
-  tmp[['__fake_data__']] <- tmp[['__weight_sdmTMB__']] <-
-    tmp[['__sdmTMB_offset__']] <- tmp[['__dcens_upr__']] <- NULL
     out_structure <- structure(list(
-    data       = tmp,
-    offset     = offset[!fd],
+    data       = data,
+    offset     = offset,
     spde       = spde,
     formula    = original_formula,
     split_formula = split_formula,
@@ -1393,10 +1386,10 @@ sdmTMB <- function(
     threshold_function = thresh[[1]]$threshold_func,
     epsilon_predictor = epsilon_predictor,
     time       = time,
-    time_lu    = lu,
+    time_lu    = time_df,
     family     = family,
     smoothers = sm,
-    response   = y_i[!fd,,drop=FALSE],
+    response   = y_i,
     tmb_data   = tmb_data,
     tmb_params = tmb_params,
     tmb_map    = tmb_map,
@@ -1457,7 +1450,7 @@ sdmTMB <- function(
 
   tmb_obj <- TMB::MakeADFun(
     data = tmb_data, parameters = tmb_params, map = tmb_map,
-    profile = if (control$profile) prof else NULL,
+    profile = control$profile,
     random = tmb_random, DLL = "sdmTMB", silent = silent)
   lim <- set_limits(tmb_obj, lower = lower, upper = upper,
     loc = spde$mesh$loc, silent = FALSE)
@@ -1468,6 +1461,7 @@ sdmTMB <- function(
   out_structure$lower <- lim$lower
   out_structure$upper <- lim$upper
 
+
   if (!do_fit) return(out_structure)
 
   if (normalize) tmb_obj <- TMB::normalize(tmb_obj, flag = "flag", value = 0)
@@ -1502,6 +1496,7 @@ sdmTMB <- function(
   }
   check_bounds(tmb_opt$par, lim$lower, lim$upper)
 
+  if (!silent) cli_inform("running TMB sdreport\n")
   sd_report <- TMB::sdreport(tmb_obj, getJointPrecision = get_joint_precision)
   conv <- get_convergence_diagnostics(sd_report)
 
@@ -1509,6 +1504,8 @@ sdmTMB <- function(
   out <- c(out_structure, list(
     model      = tmb_opt,
     sd_report  = sd_report,
+    parlist = tmb_obj$env$parList(par = tmb_obj$env$last.par.best),
+    last.par.best = tmb_obj$env$last.par.best,
     gradients  = conv$final_grads,
     bad_eig    = conv$bad_eig,
     pos_def_hessian = sd_report$pdHess))
@@ -1593,12 +1590,12 @@ parse_spatial_arg <- function(spatial) {
   spatial
 }
 
-check_irregalar_time <- function(data, time, spatiotemporal, time_varying) {
+check_irregalar_time <- function(data, time, spatiotemporal, time_varying, extra_time) {
   if (any(spatiotemporal %in% c("ar1", "rw")) || !is.null(time_varying)) {
     if (!is.numeric(data[[time]])) {
       cli_abort("Time column should be integer or numeric if using AR(1) or random walk processes.")
     }
-    ti <- sort(unique(data[[time]]))
+    ti <- sort(union(unique(data[[time]]), extra_time))
     if (length(unique(diff(ti))) > 1L) {
       missed <- find_missing_time(data[[time]])
       msg <- c(
